function [ z ] = bootfcorr2( m )
%bootfcorr2 correlate diagnoses with values with cheap bootstrap
%
% Discovering and Applying Knowledge in Clinical Databases
%  gh13@columbia.edu, DBMI, Columbia U, 2013
%
% only do bootstrap on three lags and average them
% m<-matrix of fakemrn,time,type,value
%    where type is 0 or 1 (which variable)
%     variable 0 gets normalized
%     calling fnc drops duplicate rows
%     assumes NOT have simultaneous true and false unless mean it
% z<-131x1 of 121 lags and 8 zero and 2 average stddev
%

% create bootstrap replication
m = sortrows(m); % sort by mrn,time,type,value
t = 1:size(m,1)+1; % c=get number of cases per mrn
c = diff(t([true;0~=diff(m(:,1));true]));
t = 1:size(m,1); % indices into m
tc = mat2cell(t',c); % partition t into mrn groups
n = size(c,2); % number of mrns
b = [-30 0 30]; % bootstrap iterations to run for shortened version

z2=zeros(3,25);
parfor k=1:25; % 25 bootstrap interations
    j = 1+floor(n*rand(1,n)); % bootstrap sample indices
    tc2=tc(j); % bootstrap sample
    m2 = m(vertcat(tc2{:}),:); % create m
    m2(:,1) = repli(c(j)); % new fake mrns so no dups

    for i=1:3; % just do three lags for bootstrap
        z2(i,k)=fcorr(m2,b(i));
        gsync; % not clear if needed
    end
end

z = runfcorr(m); % run main answer
z = [z;[0 0 0 0 0 0 0 0 1 -1]'*mean(std(z2,0,2))]; % append stddev
end
