function [ z ] = fcorr( m,lag )
%fcorr correlate diagnoses with values
%
% Discovering and Applying Knowledge in Clinical Databases
%  gh13@columbia.edu, DBMI, Columbia U, 2013
%
% lag (days)
% m<-matrix of fakemrn,time,type,value
%    where type is 0 or 1 (which variable)
%     variable 0 gets normalized
%     calling fnc drops duplicate rows
%     assumes NOT have simultaneous true and false unless mean it
% z<-corrcoef_corrected
%
m(:,2) = m(:,2)+lag*(m(:,3)~=0); % add time lag to diag rows
m = sortrows(m); % sort by mrn, time, value

m1 = fcorrprop(m,0); % forward propagation of continous values and times
m2=flipud(fcorrprop(flipud(m),0)); % backward propagation of continous values and times
m9=fcorrinterp(m,m1,m2,1); % interpolate continuous

m1 = fcorrprop(m,1); % forward propagation of diag values and times
m2=flipud(fcorrprop(flipud(m),1)); % backward propagation of diag values and times
m8=fcorrinterp(m,m1,m2,0); % interpolate diag

% correct continuous for mean and std dev - using cumsum
% (Runs on GPU. Runs on CPU without the gdouble.
% Will differ from mat2cell version because slight difference in 
% last digit of the running sum, which is large, translates to a large
% difference (1e-7) once the subtraction is done. Partially fix (1e-10) by
% subtracting the overall mean and adding it back if needed.)
m9c = m9;
t = 1:size(m9,1)+1; % start calculation for number of cases per mrn
b = [0~=diff(m9c(:,1)) ; true]; % b=end of each run of mrns
i = diff(t([true;b])); % i=number of cases per mrn
m9c(:,4) = m9c(:,4)-mean(m9c(:,4)); % subtract mean to improve cs accuracy
t = cumsum(m9c(:,4)); % cumulative sum of values
mean9=diff([0;t(b)])./i'; % mean for each mrn
t2 = (m9c(:,4)-mean9(repli(i))).^2; % (x-u)*2
t3 = mean(t2); % subtract mean to improve cumsum accuracy
t = cumsum(t2-t3); % cumulative sum of values (minus mean)
std9=sqrt(max(0,((t3*i')+diff([0;t(b)]))./(i'-1))); % stddev for each mrn
m9c(:,4) = (m9c(:,4)-mean9(repli(i)))./max(.0000001,std9(repli(i))); % correct

t = corrcoef([m8(:,4) m9c(:,4)]); % linear correlation after correction
z = t(2);

end

function [ z ] = fcorrprop( m1,vtype )
% fcorrprop propagate values and times
b1 = vtype==m1(:,3);
b2 = [true;0~=diff(m1(:,1))] | b1; % continuous or mrn start
t = b1(b2); % will be 0 where diag starts mrn run
b3 = b2 | ~t(cumsum(b2)); % only 1 where propagation should occur
t = m1(b3,[2 3 4]); % will only propagate diag not at start of mrn
m1(:,[2 3 4]) = t(cumsum(b3),:);
z = m1;
end

function [ z ] = fcorrinterp( m9,m1,m2,vtype )
% fcorrinterp interpopate m9 between m1 and m2
m9(:,4)=((m2(:,4).*(m9(:,2)-m1(:,2)))+(m1(:,4).*(m2(:,2)-m9(:,2))))./(m2(:,2)-m1(:,2));
b = m1(:,2)==m2(:,2); % if times same, pick one value
if any(b); m9(b,[3 4]) = m2(b,[3 4]); end % avoid empty assign for GPU
b = vtype==m1(:,3); % if lead in is negative, pick other
if any(b); m9(b,[3 4]) = m2(b,[3 4]); end
b = vtype==m2(:,3); % if follow up is negative, pick other
if any(b); m9(b,[3 4]) = m1(b,[3 4]); end
z = m9;
end

% correct continuous for mean and std dev - using cells
% (On CPU, 4 sec in 40 slower than cumsum version. This version not work
% for GPU. But may be more accurate for CPU. E.g., differs by 1e-10.)
%m9c = m9;
%t = 1:size(m9,1)+1; % i=get number of cases per mrn
%i = diff(t([true;0~=diff(m9(:,1));true]));
%m9a = mat2cell(m9(:,4),i); % partition m9 (continuous) into mrn groups
%mean9 = cellfun(@mean,m9a); % mean and stddev within each group
%std9 = cellfun(@std,m9a);
%m9c(:,4) = (m9c(:,4)-mean9(repli(i)))./max(.0000001,std9(repli(i))); % correct

% calculate correlation coefficient - GPU version, slower for CPU
%m1 = m8(:,4)-sum(m8(:,4))/size(m8,1); % x-mean(x)
%m2 = m9c(:,4)-sum(m9c(:,4))/size(m9c,1); % y-mean(y)
%t = sum(m1.*m2)/sqrt(sum(m1.^2)*sum(m2.^2)); % correlation coefficient
%z = double(t); % return GPU var to CPU
