function [ z ] = trimfcorr( m )
%trimfcorr drop duplicate rows and patients with no information
%
% Discovering and Applying Knowledge in Clinical Databases
%  gh13@columbia.edu, DBMI, Columbia U, 2013
%
% (very slow)
% m =  fakemrn, time, type, value
%
m = sortrows(m); % sort by mrn,time,type,value

% average values that are within same mrn,time,type - OFF BY 1E-12
b = [any(0~=diff(m(:,[1 2 3])),2); true]; % true at group ends
t = 1:size(m,1)+1;  % temp list
i = diff(t([true; b]));  % #cases per mrn/time/type
mv = mean(m(:,4)); % overall mean, will substract for better accuracy
t = cumsum(m(:,4)-mv); % cumulative sum of values
m = m(b,:); % new version of mat without extra rows
m(:,4) = mv+(diff([0;t(b)])./i'); % mean for each group
m(:,4) = roundn(m(:,4),-10); % get rid of cumsum noise

% drop mrns that add no information b/c only one type
m = sortrows(m,[1 3 4]); % sort by mrn,type,value
b = any(0~=diff(m(:,1)),2); % group by mrn, one short
t = 1:size(m,1)+1;  % temp list
i = diff(t([true; b; true]));  % #cases per mrn
bt = m([true;b],3)~=m([b;true],3); % true if mrn has both types
m = m(bt(repli(i)),:); % drop rows for mrns with one type

% drop mrns that add no information b/c single value for one type
b = any(0~=diff(m(:,[1 3])),2); % group by mrn/type one short
t = 1:size(m,1)+1;  % temp list
i = diff(t([true; b; true]));  % #cases per mrn/type
bt = m([true;b],4)~=m([b;true],4); % true if mrn/type has at least 2 values
bt = all(reshape(bt,[2 round(size(bt)/2)]),1)'; % both types within each mrn
i = sum(reshape(i,[2 round(size(i,2)/2)]),1)'; % both types within each mrn
m = m(bt(repli(i')),:); % drop rows for mrns with one value per type

z = sortrows(m); % sort by mrn,time,type,value

end

% convert potalddup to fcorr format
%pp=potalddup(:,[1 2 3 3]);
%pp(:,3)=0>pp(:,4);
%pp(1==pp(:,3),4)=-1-pp(1==pp(:,3),4);

% old trim dups
%m = sortrows(m); % sort by mrn, time, value
%m = m([true;~all(0==diff(m),2)],:); % drop exact duplicate rows (not in APL)
%b = all((0==diff(m(1:end,[1 2]))),2); % same mrn and time in consec rows
%b2 = [true;~(b&(m(1:end-1,3)==-2 & m(2:end,3)==-1))]; % -2 and -1
%m = m(b2,:); % drop rows with -1 when a -2 exists at the same time
%z=m;

% average values that are within same mrn,time,type - SLOW
%b = [true; any(0~=diff(m(:,[1 2 3])),2)]; % true at group boundaries
%t = 1:size(m,1)+1;  % temp list
%i = diff(t([b;true]));  % #cases per mrn/time/type
%v = mat2cell(m(:,4),i); % partition value into mrn/time/type groups
%m = m(b,:); % reduce m to unique mrn/time/types
%m(:,4) = cellfun(@mean,v); % insert averaged values

% average values that are within same mrn,time,type - SLOWER
%first = 1;
%mrn = m(1,1:3);
%b = false(size(m,1),1);
%for i=2:size(m,1);
%    if any(mrn ~= m(i,1:3));
%        m(i-1,4) = mean(m(first:i-1,4));
%        b(i-1) = true;
%        mrn = m(i,1:3);
%        first = i;
%    end
%end
%m(end,4) = mean(m(first:end,4));
%b(end) = true;
%m = m(b,:);        

